/*** This Programs/Libraries are (C)opyright by S. Krahmer.
 *** You may use it under the terms of the GPL. You should have
 *** already received the file COPYING that shows you your rights. If not
 *** you can get it at http://www.cs.uni-potsdam.de/homepages/students/linuxer
 *** the logit-package. You will also find some other nice utillities there.
 ***  
 *** THERE IS ABSOLUTELY NO WARRANTY. SO YOU USE IT AT YOUR OWN RISK.
 *** IT WAS WRITTEN IN THE HOPE THAT IT WILL BE USEFULL. I'M NOT RESPONSIBLE
 *** FOR ANY DAMAGE YOU MAYBE GET DUE TO USING MY PROGRAMS.
 ***/

#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <popt.h>
#include <signal.h>
#include <memory>

#include "lncf.h"
#include "service.h"
#include "misc.h"

int lockout = 0;

#define INTRO "\nList Not Closed Files by S. Krahmer -- Get lattest version from\n"\
              "http://www.cs.uni-potsdam.de/homepages/students/linuxer\n\n"

void help_her()
{
	printf("Wrong arguments. 'lncf -?' for help.\n");
	exit(0);
}
    

int main(int argc, char **argv)
{
   	poptContext pc;
        int list_all = 0, pid_only = 0, read_fd = -1, write_fd = -1, 
	    self = 0, interactive = -1, ssh = -1;
	char c;
	char *infile = NULL, *outfile = NULL, *errfile = NULL;
	
        struct poptOption optTable[] = {
		{ "all", 'a', POPT_ARG_NONE, &list_all, 1, 
		  "look at all processes" , NULL
		},
        	{ "pid", 'p', POPT_ARG_INT, &pid_only, 1, 
		  "only handle this pid", "PID"
		},
		{ "read", 'r', POPT_ARG_INT, &read_fd, 1,
		  "read from this descriptor; requires '-p'", "fd" 
		},
		{ "write",'w', POPT_ARG_INT, &write_fd, 1,
		  "write into this descriptor; requires '-p'", "fd" 
		},
		{ "self", 's', POPT_ARG_NONE, &self, 1, 
		  "only watch ourself", NULL
		},
		{ "out", 'o', POPT_ARG_STRING, &outfile, 1, 
		  "use this file for stdout", "file" 
		},
		{ "in", 'i', POPT_ARG_STRING, &infile, 1,
		  "use this file for stdin", "file"
		},
		{ "err", 'e', POPT_ARG_STRING, &errfile, 1, 
		  "use this file for stderr", "file"
		},
		{ "interactive", 'I', POPT_ARG_INT, &interactive, 1,
		  "hijack session via interactive-usermode; needs '-p'", "fd"
		},
		{ "ssh", 'S', POPT_ARG_INT, &ssh, 1,
		  "insert commands into a ssh connection; needs '-p'", "fd"
		},
		{ "lock", 'l', POPT_ARG_NONE, &lockout, 1,
		  "prevent the hijacked process from taking back control over fd "\
		  "while hijacking", 
		   NULL
		},
		POPT_AUTOHELP
		{ NULL, 0, 0, NULL, 0, NULL, NULL}
	};

	if (argc < 2)
		help_her();

	signal(SIGINT, sighandler);

	cout<<INTRO;

        pc = poptGetContext(NULL, argc, argv, optTable, 0);
	c = poptGetNextOpt(pc);
	while ((c = poptGetNextOpt(pc)) >= 0);
	
	if (c < -1) 
		help_her();
		
	// stupid freopen() bug workaround ...
	if (infile) {
		close(0);
		if (open(infile, O_RDONLY) != 0) {
			perror("eopen");
			exit(errno);
		}
	}
	if (outfile) {
		close(1);
		if (open(outfile, O_RDWR|O_CREAT, 0600) != 1) {
			perror("open");
			exit(errno);
		}
	}
	if (errfile) {
		close(2);
		if (open(errfile, O_RDWR|O_CREAT, 0600) != 3) {
			perror("open");
			exit(errno);
		}
	}
	// if '-l' specified, fire up global variable with right value
	if (lockout > 0) {
		lockout = HJS_LOCK;
	}
	
	// the lines below might look ugly, but thats the way
        if (list_all) {
           	do_all();
        } else if (self) {
		cout<<"Watching myself ("<<getpid()<<")\n";
           	do_pid(getpid());
        } else if (read_fd != -1) {
		cout<<"Try to read from filedescriptor "<<read_fd<<" of process "<<pid_only<<endl;
		service *s = new service(pid_only, read_fd, lockout);
		s->read();
		delete s;
	} else if (write_fd != -1) {
		cout<<"Try to write to filedescriptor "<<write_fd<<" of process "<<pid_only<<endl;
		service *s = new service(pid_only, write_fd, lockout);
		s->write();
		delete s;
	} else if (interactive != -1) {
		cout<<"Try to interactively hijack a filedescriptor.\n"\
		      "Please be sure to use an bidirectional (opened for read AND write) fd.\n";
		service *s = new service(pid_only, interactive, lockout);
		s->interactive();
		delete s;
	} else if (ssh != -1) {
		cout<<"OK. Inserting commands in strange stdin. Please use stdin (0) only!\n";
		service *s = new service(pid_only, ssh, lockout);
		s->ssh();
		delete s;
	} else if (pid_only) {
		do_pid(pid_only);
	}
	poptFreeContext(pc);
        return 0;
}
